program monte_carlo_integration
!
!       peform an N dimensional integration of exp(-t**2/2)
!       over the square [0,1] x [0,1] x ... x [0,1]
!       for N dimensions using a Monte Carlo scheme.
!
!
!=========================================================================
!
!  This code is part of the Quetzal Computational Associates Fortran 90
!  Benchmark Suite.
!
!  Copyright (c) 1994 by Quetzal Computational Associates.
!
!  Permission to copy and use without fee all or part of the
!  publications or computer codes in this suite is granted provided 
!  that copies are not made or distributed for direct commercial
!  advantage, this copyright notice and the title of the codes
!  or publications and the fact that they originated in this
!  benchmark suite appear, and notice is given that copying is
!  by permission of Quetzal Computational Associates.  For
!  publications that have been published in journals, the name
!  of the publication and date should appear.  To copy otherwise,
!  or to republish, requires a fee and/or specific permission.
!  Authors of codes in this benchmark suite are exempt from the terms of
!  this copyright notice for their codes only.  Address any
!  questions about this copyright notice to:
!
!         John K. Prentice
!         Quetzal Computational Associates
!         3200 Carlisle N.E.
!         Albuquerque, NM   87110-1664
!         USA
!
!         Phone:  505-889-4543
!         Fax:    505-889-4598
!         E-mail: quetzal@aip.org
!
!=========================================================================
!
    implicit none
    integer, parameter :: problem_dimension = 10 
    integer, parameter :: number_of_trys = 6 
!
    integer :: k, number_of_integration_points
    real :: integral, exact_answer
    real, dimension(:,:), allocatable :: integration_points 
    real, dimension(:), allocatable :: argument
    real :: exact_answer_in_1d = 0.85562439
    integer, dimension(number_of_trys) :: try_size 
!
    data try_size/9000, 100, 60000, 423, 100000, 2533/ 
!
!       loop over different trys
!
    do k = 1, number_of_trys 
      number_of_integration_points = try_size(k) 
!
!       allocate memory 
!
      allocate (integration_points(number_of_integration_points,problem_dimension))
      allocate (argument(number_of_integration_points))
!
!       randomly calculate integration points
!
      call gleich
!
!        do Monte Carlo integration
!
      argument = sum(0.5*integration_points**2,dim=2) 
      integral = sum(exp(-argument)) / real(number_of_integration_points)
!
      exact_answer = exact_answer_in_1d**problem_dimension 
      print *, ' ' 
      print *, 'n = ', problem_dimension, ' number of points = ', &
     &      number_of_integration_points 
      print *, 'exact answer       = ', exact_answer 
      print *, 'Monte Carlo answer = ', integral 
!
      deallocate (integration_points)
      deallocate (argument)
!
    end do 
!
contains
!
    subroutine gleich
!
!        this routine by t. t. warnock, cray research analyist at lanl
!        F77 version written about 1980
!
!
    implicit none
!
    real, dimension(number_of_integration_points,problem_dimension) :: bottom, top, radish_value
    integer, dimension(number_of_integration_points,problem_dimension) :: radix_value, spin_value, &
&                                                                         quo, errors
    integer :: kount
!
    integer, dimension(25) :: radix, spin 
    real, dimension(25) :: radish 
!
    data radix/2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 59,&
&              61, 67, 71, 73, 79, 83, 89, 97/ 
    data radish/2., 3., 5., 7., 11., 13., 17., 19., 23., 29., 31., 37., 41.,  &
&               43., 47., 53., 59., 61., 67., 71., 73., 79., 83., 89., 97./ 
    data spin/1, 2, 1, 5, 3, 8, 2, 7, 18, 11, 18, 3, 17, 24, 40, 15, 40, 49,  &
&             12, 30, 40, 70, 9, 39, 82/ 
!
    top = 0. 
    bottom = 1. 
    do kount = 1,number_of_integration_points
        radish_value(kount,1:problem_dimension) = radish(1:problem_dimension)
        spin_value(kount,1:problem_dimension) = spin(1:problem_dimension)
        radix_value(kount,1:problem_dimension) = radix(1:problem_dimension)
        quo(kount,1:problem_dimension) = kount
    enddo
!
10  where (quo /= 0)
        top = top * radish_value + mod(quo * spin_value, radix_value)
        bottom = bottom * radish_value
        quo = quo / radix_value
        errors = 1
    elsewhere
        errors = 0
    end where
    if (sum(errors) > 0) go to 10
!
    integration_points = top/bottom 
!
    end subroutine gleich 
!
end program monte_carlo_integration
